//-----------------------------------------------------------------------------
// File:     HelloMath.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  Tests out various mathematical functions
//-----------------------------------------------------------------------------
// The following are standard C/C++ header files.
// If a filename is enclosed inside < >  it means the header file is in the Include directory.
// If a filename is enclosed inside " "  it means the header file is in the current directory.
#include <ctype.h>      // Character Types
#include <math.h>       // Mathematical Constants
#include <stdarg.h>     // Variable Argument Lists
#include <stdio.h>      // Standard Input/Output Functions
#include <stdlib.h>     // Utility Functions
#include <string.h>     // String Operations
#include <signal.h>     // Signals (Contol-C + Unix System Calls)
#include <setjmp.h>     // Nonlocal Goto (For Control-C)
#include <time.h>       // Time and Date information
#include <assert.h>     // Verify Program Assertion
#include <errno.h>      // Error Codes (Used in Unix system())
#include <float.h>      // Floating Point Constants
#include <limits.h>     // Implementation Constants
#include <stddef.h>     // Standard Definitions
#include <exception>    // Exception handling (e.g., try, catch throw)
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
bool  WriteDoubleToFile( double x, int precision, FILE *fptr );


//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{
   // Write  " Hello math!"  to the screen and then put a newline
   printf( " Hello math!\n" );

   // Write the number of command line arguments to the screen.
   // Write each of the command line arguments on a separate line.
   // Note: The first command line argument may be the name of the program
   // In Microsoft Windows, dragging and dropping files onto the executable
   // usually results in multiple command line arguments (the names of the files)
   printf( "\n The number of command line arguments is %d\n", numberOfCommandLineArguments );
   for( int i = 0;  i < numberOfCommandLineArguments;  i++ )
      printf( "\n Command line argument %d is:\n %s", i, arrayOfCommandLineArguments[i] );

   // Calculate the sine of 1.0 radian (1 radians is approximately 57.3 degrees) plus other stuff
   double x = sin(1.0) + cos(1.0) + tan(1.0) +  asin(0.7) + sqrt(4.2) + pow(3.3,0.8)
            + log(1.5) + log10(4.2) + exp(1.4) + sinh(0.3) + 2*3.2 + rand();

   // Writes the result to the screen (stdout)
   printf( "\n\n The computed number is: " );
   WriteDoubleToFile( x, 7, stdout );

   // Keep the screen displayed until the user presses the Enter key
   printf( "\n\n Press  Enter  to terminate the program: " );
   int key = getchar();

   // A normal program exit returns 0 (other return values usually signal an error)
   return 0;
}


//-----------------------------------------------------------------------------
bool  WriteDoubleToFile( double x, int precision, FILE *fptr )
{
   // Ensure the precision (number of digits in the mantissa after the decimal point) makes sense.
   // Next, calculate the field width so it includes one extra space to the right of the number.
   if( precision < 0 || precision > 17 ) precision = 15;
   int fieldWidth = precision + 8;

   // Create the format specifier and print the number
   char format[20];
   sprintf( format, " %%- %d.%dE", fieldWidth, precision );
   return fprintf( fptr, format, x ) > 0;
}
